import type * as JSONSchema from "./json-schema.js";
import { $ZodRegistry } from "./registries.js";
import type * as schemas from "./schemas.js";
interface JSONSchemaGeneratorParams {
    /** A registry used to look up metadata for each schema. Any schema with an `id` property will be extracted as a $def. */
    metadata?: $ZodRegistry<Record<string, any>>;
    /** The JSON Schema version to target.
     * - `"draft-2020-12"` — Default. JSON Schema Draft 2020-12
     * - `"draft-7"` — Default. JSON Schema Draft 7 */
    target?: "draft-7" | "draft-2020-12";
    /** How to handle unrepresentable types.
     * - `"throw"` — Default. Unrepresentable types throw an error
     * - `"any"` — Unrepresentable types become `{}` */
    unrepresentable?: "throw" | "any";
    /** Arbitrary custom logic that can be used to modify the generated JSON Schema. */
    override?: (ctx: {
        zodSchema: schemas.$ZodType;
        jsonSchema: JSONSchema.BaseSchema;
    }) => void;
    /** How to handle pipes. Pipes contain an input and output schema.
     * - `"output" — Default. Convert the output schema.
     * - `"input"` — Convert the input schema. */
    pipes?: "input" | "output";
}
interface ProcessParams {
    schemaPath: schemas.$ZodType[];
    path: (string | number)[];
    initial?: Seen | undefined;
}
interface EmitParams {
    /** How to handle cycles.
     * - `"ref"` — Default. Cycles will be broken using $defs
     * - `"throw"` — Cycles will throw an error if encountered */
    cycles?: "ref" | "throw";
    reused?: "ref" | "inline";
    /** A function used to convert `id` values to URIs to be used in *external* $refs.
     *
     * Default is `(id) => id`.
     */
    external?: {
        registry: $ZodRegistry<{
            id?: string | undefined;
        }>;
        uri: (id: string) => string;
        defs: Record<string, JSONSchema.BaseSchema>;
    } | undefined;
}
interface Seen {
    schema: JSONSchema.BaseSchema;
    cached: JSONSchema.BaseSchema;
    count: number;
    cycle?: (string | number)[] | undefined;
    defId?: string | undefined;
}
export declare class JSONSchemaGenerator {
    metadataRegistry: $ZodRegistry<Record<string, any>>;
    target: "draft-7" | "draft-2020-12";
    unrepresentable: "throw" | "any";
    override: (ctx: {
        zodSchema: schemas.$ZodType;
        jsonSchema: JSONSchema.BaseSchema;
    }) => void;
    pipes: "input" | "output";
    counter: number;
    seen: Map<schemas.$ZodType, Seen>;
    constructor(params?: JSONSchemaGeneratorParams);
    process(schema: schemas.$ZodType, _params?: ProcessParams): JSONSchema.BaseSchema;
    emit(schema: schemas.$ZodType, _params?: EmitParams): JSONSchema.BaseSchema;
}
interface ToJSONSchemaParams extends Omit<JSONSchemaGeneratorParams & EmitParams, never> {
}
interface RegistryToJSONSchemaParams extends Omit<JSONSchemaGeneratorParams & EmitParams, never> {
    uri?: (id: string) => string;
}
export declare function toJSONSchema(schema: schemas.$ZodType, _params?: ToJSONSchemaParams): JSONSchema.BaseSchema;
export declare function toJSONSchema(registry: $ZodRegistry<{
    id?: string | undefined;
}>, _params?: RegistryToJSONSchemaParams): {
    schemas: Record<string, JSONSchema.BaseSchema>;
};
export {};
